var _ = require('lodash');

var path = require('path');
var rfr = require('rfr');

var Parser = require('./lib/parser');

/**
 * Appo provides all services necessary to create a command line tool
 */
class Appo {
  constructor() {

    // TODO provide documentation for this constructor
    this._name = null;
    this._fullName = '';

    this._version = '';

    this._groups = [];
    this._commands = [];
    this._options = [];

    this._help = null;
    this._description = null;

    this.argv = null;

    this._parent = null;
  }

  /**
   * name sets the name of the program
   */
  name(_name) {
    /**
     * TODO if name is empty exit process
     */
    this._name = _name;
    return this;
  }

  /**
   * setParent sets the parent of obj
   */
  setParent(_parent) {
    this._parent = _parent;
  }

  /**
   * getFullName returns all parents of the current object
   */
  getFullName() {

    var self = this;
    var fullName = self._name;

    /**
     * while self has a setParent prepend the parents name to fullName and set new self to matched setParent
     */
    while (_.has(self, '_parent') && self._parent) {
      fullName = `${self._parent._name} ${fullName}`;

      if (self._parent) {
        self = self._parent;
      }

      if (!self._parent) {
        break;
      }

    }

    return fullName;
  }

  /**
   * version sets the version of the program
   */
  version(version) {
    if (!version) {
      /**
       * TODO if version is empty exit process
       */
    }
    this._version = version;
    return this;
  }

  /**
   * help sets the help of the extended object
   */
  help(_help) {
    this._help = _help;
    return this;
  }

  /**
   * description sets the description of the current object
   */
  description(_description) {
    this._description = _description;
    return this;
  }

  /**
   * add new group to the current object
   */
  group(group) {
    group.type = 'group';
    group.setParent(this);
    this._groups.push(group);
    return this;
  }

  /**
   * add new command to the current object
   */
  command(command) {
    command.type = 'command';
    command.setParent(this);
    this._commands.push(command);
    return this;
  }

  /**
   * add new flag to the current object
   */
  option(_option) {
    _option.type = 'option';
    this._options.push(_option);
    return this;
  }

  /**
   * findCommand checks if the first entry in the argv array matches a command of the current object
   */
  findCommand(argv) {
    return _.find(this._commands, function (command) {
      return command._name === _.first(argv._);
    });
  }

  /**
   * findGroup checks if the first entry in the argv array matches a group of the current object
   */
  findGroup(argv) {
    return _.find(this._groups, function (group) {
      return group._name === _.first(argv._);
    })
  }

  /**
   * parse parses the current argv and either executes the action or logs the help
   */
  parse(argv) {
    Parser.parse.call(this, require('minimist')(argv.slice(2)));
  }

}

module.exports = Appo;