var _ = require('lodash');

var Logger = require('../logger');

class Parser {
  constructor() {
  }

  /**
   * parse takes a class and calls it's init method
   */
  static parse(argv) {

    var command = null;
    var group = null;

    /**
     * try to find a valid command
     */
    if (argv._.length > 0) {
      command = this.findCommand(argv);

      /**
       * try to find a valid command
       */
      if (!command) {
        group = this.findGroup(argv)
      }

      /**
       * try to find a valid group
       */
      if (group) {
        argv._.shift();

        if (argv._.length > 0) {
          /**
           * set new context to match and call parse again
           * to prevent further execution call return
           */
          Parser.parse.call(group, argv);
          return;
          // TODO promise
        } else {
          // TODO
          Logger.logGroup(group, true);
          process.exit();
        }

      }

      /**
       * if a command was found, check if required options are set and execute it's action
       */
      if (command) {
        argv._.shift();

        if (command._options.length > 0) {

          /**
           * make a copy of argv, argv should not be touched
           */
          let tmpArgv = _.cloneDeep(argv);
          delete tmpArgv._;

          /**
           * filter all option, which are required, and map the results to an array of names
           */
          var requiredOptionNames = command._options.filter(function (option) {
            return option._required;
          }).map(function (option) {
            return option._name;
          });

          /**
           * filter all missing required option names,
           * argv contains the given options, it's keys are transformed to an array
           */
          var missingRequiredOptionNames = _.difference(requiredOptionNames, _.map(tmpArgv, function (value, key) {
            return key;
          }));

          /**
           * if the required option names array length is greater than zero,
           * some required option names are not provided by the user
           */
          if (missingRequiredOptionNames.length > 0) {

            /**
             * filter out the missing options by their name (requiredOptionNames contains now only the missing options)
             */
            var missingOptions = _.map(missingRequiredOptionNames, function (name) {
              return _.find(command._options, function (option) {
                return option._name === name;
              })
            });

            /**
             * log the commands manual
             */
            Logger.logCommand(command, true);
            /**
             * log all missing choices
             */
            Logger.logMissingOptions(missingOptions);
            console.log();

            process.exit();
          }

        }

        command._action(argv)
          .then(function () {
            process.exit();
          })
          .catch(function () {
            command.logCommand(command, true);
            process.exit();
          });
        return;
      }

      /**
       * if neither a group nor a command was found log the programs manual
       */
      if (!group && !command) {
        Logger.log(this, true);
        process.exit();
      }


    }

    var tmpArgv = _.cloneDeep(argv);
    delete tmpArgv._;

    if (_.keys(tmpArgv).length === 0) {
      Logger.log(this);
      process.exit();
    }

    if (this._options.length) {

      if (!this.optionActions) {
        Logger.log(this);
        process.exit();
      }

      this.optionActions(argv)
        .then(function () {
          process.exit();
        })
        .catch(function () {
          process.exit();
        });
    }

    process.exit();

  }

}

module.exports = Parser;